#!/usr/bin/env perl

# LSG.pm - Lumidify Site Generator
# Written by lumidify <nobody@lumidify.org>
# Last updated: 2019-08-21
#
# To the extent possible under law, the author has dedicated
# all copyright and related and neighboring rights to this
# software to the public domain worldwide. This software is
# distributed without any warranty.
#
# You should have received a copy of the CC0 Public Domain
# Dedication along with this software. If not, see
# <http://creativecommons.org/publicdomain/zero/1.0/>.

# Note: cross-platform path processing is used wherever possible, but
# other parts won't work properly anyways if the path separator isn't /.
# Good that nobody important uses any OS on which that's the case.

package LSG;
use strict;
use warnings;
use LSG::Config;
use LSG::Template;
use LSG::UserFuncs;
use LSG::Metadata;
use LSG::Generate;
use Data::Dumper;

# FIXME: don't just chdir into $path, in case that messes up anything
# the calling script wanted to do afterwards
sub init {
	my $path = shift;
	chdir($path) or die "Unable to access directory \"$path\": $!\n";
	LSG::Config::init_config("config.ini", "modified_dates");
	LSG::Template::init_templates();
	LSG::Metadata::init_metadata();
	LSG::UserFuncs::init_userfuncs();
}

sub generate_site {
	LSG::Generate::gen_files();
	LSG::Generate::delete_obsolete();
	LSG::Config::write_modified_dates("modified_dates");
}

1;
