#!/usr/bin/env perl

# LSG::Config - configuration for the LSG
# Written by lumidify <nobody@lumidify.org>
# Last updated: 2019-08-21
#
# To the extent possible under law, the author has dedicated
# all copyright and related and neighboring rights to this
# software to the public domain worldwide. This software is
# distributed without any warranty.
#
# You should have received a copy of the CC0 Public Domain
# Dedication along with this software. If not, see
# <http://creativecommons.org/publicdomain/zero/1.0/>.

package LSG::Config;
use strict;
use warnings;
use utf8;
use open qw< :encoding(UTF-8) >;

use Exporter qw(import);
our @EXPORT_OK = qw($config);

# Yes, I know this isn't just used for real config
our $config;

sub read_modified_dates {
	my $path = shift;
	my %dates = (pages => {}, templates => {});
	if (!-f $path) {
		print(STDERR "Unable to open \"$path\". Using empty modified_dates.\n");
		return \%dates;
	}
	open (my $fh, "<", $path) or die "Unable to open $path: $!\n";
	foreach (<$fh>) {
		chomp;
		my @fields = split(" ", $_, 2);
		my $date = $fields[0];
		my $filename = $fields[1];
		if ($filename =~ /\Apages\//) {
			$dates{"pages"}->{substr($filename, 6)} = $date;
		} elsif ($filename =~ /\Atemplates\//) {
			$dates{"templates"}->{substr($filename, 10)} = $date;
		} else {
			die "Invalid file path \"$filename\" in \"$path\".\n";
		}
	}
	close($fh);
	return \%dates;
}

sub write_modified_dates {
	my $path = shift;
	open(my $fh, ">", $path) or die "Unable to open \"$path\": $!\n";
	foreach my $pageid (keys %{$config->{"metadata"}}) {
		foreach my $lang (keys %{$config->{"metadata"}->{$pageid}->{"modified"}}) {
			print($fh $config->{"metadata"}->{$pageid}->{"modified"}->{$lang} . " pages/$pageid.$lang\n");
		}
	}
	foreach my $template (keys %{$config->{"templates"}}) {
		print($fh $config->{"templates"}->{$template}->{"modified"} . " templates/$template\n");
	}
	close($fh);
}

sub read_config {
	my $path = shift;
	my %config;
	open (my $fh, "<", $path) or die "Unable to open $path: #!\n";
	my $section = "";
	foreach (<$fh>) {
		chomp;
		if ($_ eq "") {
			$section = "";
			next;
		}
		if (/^\[(.*)\]$/) {
			$section = $1;
			next;
		}
		my ($key, $value) = split("=", $_, 2);
		if ($value =~ /:/) {
			my @value = split(":", $value);
			$value = \@value;
		}
		if ($section) {
			$config{$section}->{$key} = $value;
		} else {
			$config{$key} = $value;
		}
	}
	close($fh);
	return \%config;
}

sub init_config {
	my ($config_path, $modified_path) = @_;
	$config = read_config($config_path);
	$config->{"modified_dates"} = read_modified_dates($modified_path);
}

1;
