#!/usr/bin/env perl

#TODO: template - func processed once and func processed for each page

# LSG::UserFuncs - user functions for the LSG (called from templates and markdown files)
# Written by lumidify <nobody@lumidify.org>
# Last updated: 2019-08-21
#
# To the extent possible under law, the author has dedicated
# all copyright and related and neighboring rights to this
# software to the public domain worldwide. This software is
# distributed without any warranty.
#
# You should have received a copy of the CC0 Public Domain
# Dedication along with this software. If not, see
# <http://creativecommons.org/publicdomain/zero/1.0/>.

package LSG::UserFuncs;
use strict;
use warnings;
use utf8;
use open qw< :encoding(UTF-8) >;
use LSG::Config qw($config);
use LSG::Misc;

# FIXME: maybe also pass line for better error messages
# Module arguments:
# 1:  page id in %fm
# 2:  page language
# 3-: other args (e.g. for func call)

sub sort_books {
	my $pageid = shift;
	my $lang = shift;
	my $sort_by = shift;
	my $create_subheadings = shift;
	if (!$sort_by) {die "ERROR: not enough arguments to function call in $pageid\n"}
	my $output = "";
	my %tmp_md = ();
	foreach my $id (keys %{$config->{"metadata"}}) {
		if ($config->{"metadata"}->{$id}->{"dirname"} eq "books") {
			$tmp_md{$id} = $config->{"metadata"}->{$id};
			if (!exists($config->{"metadata"}->{$id}->{$lang}->{$sort_by})) {
				die "ERROR: $pageid: can't sort by \"$sort_by\"\n";
			}
		}
	}
	my $current = "";
	foreach my $id (sort {$tmp_md{$a}->{$lang}->{$sort_by} cmp $tmp_md{$b}->{$lang}->{$sort_by} or
	                      $tmp_md{$a}->{$lang}->{"title"} cmp $tmp_md{$b}->{$lang}->{"title"}} (keys %tmp_md)) {
		if ($create_subheadings && $create_subheadings eq "true" && $current ne $tmp_md{$id}->{$lang}->{$sort_by}) {
			$current = $tmp_md{$id}->{$lang}->{$sort_by};
			$output .= "<h3>$current</h3>\n";
		}
		my $rel_lnk = LSG::Misc::gen_relative_link("$lang/$pageid", "$lang/$id.html");
		$output .= "<p><a href=\"$rel_lnk\">" . $tmp_md{$id}->{$lang}->{"title"} . "</a></p>\n";
	}

	return $output;
}

sub gen_lang_selector {
	my $pageid = shift;
	my $lang = shift;
	my $output = "<ul>\n";
	foreach my $nav_lang (keys %{$config->{"langs"}}) {
		if ($nav_lang ne $lang) {
			my $url = LSG::Misc::gen_relative_link("$lang/$pageid", "$nav_lang/$pageid.html");
			$output .= "<li><a href=\"$url\">" . $config->{"langs"}->{$nav_lang} . "</a></li>\n";
		}
	}
	$output .= "</ul>";

	return $output;
}

sub gen_nav {
	my $pageid = shift;
	my $lang = shift;
	# Don't print <ul>'s so extra content can be added in template
	#my $output = "<ul>\n";
	my $output = "";
	my @nav = @{$config->{"nav"}};
	# Not necessary because of direction: rtl in style
	#if ($lang_dirs{$lang} eq "rtl") {
	#	@nav = reverse(@nav);
	#}
	foreach my $nav_page (@nav) {
		my $title = $config->{"metadata"}->{$nav_page}->{$lang}->{"title"};
		my $url = LSG::Misc::gen_relative_link("$lang/$pageid", "$lang/$nav_page.html");
		$output .= "<li><a href=\"$url\">$title</a></li>\n";
	}
	#$output .= "</ul>";

	return $output;
}

sub gen_relative_link {
	my ($pageid, $lang, $link) = @_;
	return LSG::Misc::gen_relative_link("$lang/$pageid", $link);
}

sub init_userfuncs {
	$config->{"funcs"}->{"gen_lang_selector"} = \&gen_lang_selector;
	$config->{"funcs"}->{"sort_books"} = \&sort_books;
	$config->{"funcs"}->{"gen_nav"} = \&gen_nav;
	$config->{"funcs"}->{"gen_relative_link"} = \&gen_relative_link;
}

1;
